using UnityEngine;
using System.Collections;

public class Player : MonoBehaviour
{
	TextMesh textMesh;
	Vector3 targetPosition;
	Vector3 moveDirection;
	bool hasTarget;
	bool canMove;
	bool showingText;
	float textDelay;
	float textDuration;

	// Use this for initialization
	void Start ()
	{
		textMesh = GetComponentInChildren<TextMesh>();
		targetPosition = Vector3.zero;
		hasTarget = false;
		canMove = true;
		showingText = false;
	}
	
	// Update is called once per frame
	void Update ()
	{
		textMesh.transform.LookAt(Camera.main.transform);
		textMesh.transform.eulerAngles = new Vector3(26.56f, 26.56f, textMesh.transform.eulerAngles.z);
		if ( showingText )
		{
			textMesh.renderer.enabled = true;
		}
		else
		{
			textMesh.renderer.enabled = false;
		}

		// Lock X/Z rotation and Y position
		transform.position = new Vector3(transform.position.x, 1, transform.position.z);
		// transform.eulerAngles = new Vector3(0, transform.eulerAngles.y, 0);

		// Distance to our destination
		float distance = Vector3.Distance(targetPosition, transform.position);

		// Handle rotation
		if ( canMove && targetPosition != Vector3.zero && distance >= 3 )
		{
			Quaternion rotation = Quaternion.LookRotation(targetPosition - transform.position);
			rotation.eulerAngles = new Vector3(0, rotation.eulerAngles.y, 0);
			transform.rotation = Quaternion.Lerp(transform.rotation, rotation, 10 * Time.deltaTime);
		}

		// Handle movement
		
		if ( hasTarget && canMove && targetPosition != Vector3.zero && distance >= 3 )
		{
			transform.Translate(Vector3.forward * 6 * Time.deltaTime);
			transform.eulerAngles = new Vector3(transform.eulerAngles.x, transform.eulerAngles.y, Mathf.Cos(Time.time * 6f) * 15);

			if ( distance <= 3 )
			{
				targetPosition = Vector3.zero;
				rigidbody.velocity = Vector3.zero;
				rigidbody.angularVelocity = Vector3.zero;
				hasTarget = false;
			}
		}

		// Make sure we don't slide or move if we're not supposed to be moving
		if ( distance <= 3 || !hasTarget || !canMove )
		{
			rigidbody.velocity = Vector3.zero;
			rigidbody.angularVelocity = Vector3.zero;
		}
	}

	public void PlaySound()
	{
		if ( canMove )
		{
			audio.volume = 2f;
			audio.Play();
		}
	}

	public void MoveToPoint(Vector3 newTarget)
	{
		targetPosition = newTarget;
		hasTarget = true;
	}

	public void SetTargetState(bool state)
	{
		hasTarget = state;
	}

	public void SetMoveState(bool state)
	{
		canMove = state;
	}

	public void UpdateText(string text)
	{
		if ( !showingText )
		{
			textMesh.text = text;
			StartCoroutine(DelayText(textDelay));
		}
	}

	public void SetTextDelay(float wait)
	{
		textDelay = wait;
	}

	public void SetTextDuration(float wait)
	{
		textDuration = wait;
	}

	IEnumerator DelayText(float wait)
	{
		yield return new WaitForSeconds(wait);
		showingText = true;
		StartCoroutine(RemoveText(textDuration));
	}

	IEnumerator RemoveText(float wait)
	{
		yield return new WaitForSeconds(wait);
		showingText = false;
	}

	void OnCollisionEnter (Collision collision)
	{
		if ( collision.gameObject.tag == "Sheep" && Camera.main.GetComponent<GameMenu>().gameState < 3 )
		{
			gameObject.SendMessage("SetTextDelay", 0f);
			gameObject.SendMessage("SetTextDuration", 2.5);
			gameObject.SendMessage("UpdateText", "I'm sorry sheep!");

			collision.gameObject.SendMessage("SetTextDelay", 2);
			collision.gameObject.SendMessage("SetTextDuration", 2.5);
			collision.gameObject.SendMessage("UpdateText", "It's okay peach!");
		}
	}
}
