using UnityEngine;
using System.Collections;

public class Sheep : MonoBehaviour
{
	public float minFollowDistance = 8;
	public float maxFollowDistance = 10;
	public int talkState = 0;

	TextMesh textMesh;
	Vector3 targetPosition;
	GameObject Player;

	bool hasTarget;
	bool canMove;
	bool showingText;

	float moveSpeed = 5;
	float textDelay;
	float textDuration;

	// Use this for initialization
	void Start ()
	{
		talkState = 0;

		textMesh = GetComponentInChildren<TextMesh>();
		targetPosition = Vector3.zero;
		Player = GameObject.Find("Player");

		hasTarget = false;
		canMove = false;
		showingText = false;
	}
	
	// Update is called once per frame
	void Update ()
	{
		textMesh.transform.LookAt(Camera.main.transform);
		textMesh.transform.eulerAngles = new Vector3(26.56f, 26.56f, textMesh.transform.eulerAngles.z);
		if ( showingText )
		{
			textMesh.renderer.enabled = true;
		}
		else
		{
			textMesh.renderer.enabled = false;
		}

		// Lock X/Z rotation and Y position
		transform.position = new Vector3(transform.position.x, 1, transform.position.z);
		transform.eulerAngles = new Vector3(0, transform.eulerAngles.y, 0);

		float distance;
		if ( targetPosition != Vector3.zero )
		{
			distance = Vector3.Distance(targetPosition, transform.position);
		}
		else
		{
			distance = Vector3.Distance(Player.transform.position, transform.position);
		}

		// Handle rotation
		if ( canMove )
		{
			Quaternion rotation;
			if ( targetPosition != Vector3.zero )
			{
				rotation = Quaternion.LookRotation(targetPosition - transform.position);
			}
			else
			{
				rotation = Quaternion.LookRotation(Player.transform.position - transform.position);
			}

			rotation.eulerAngles = new Vector3(0, rotation.eulerAngles.y, 0);
			transform.rotation = Quaternion.Lerp(transform.rotation, rotation, 10 * Time.deltaTime);

			if ( distance >= maxFollowDistance )
			{
				hasTarget = true;
			}
			else
			{
				rigidbody.velocity = Vector3.zero;
				rigidbody.angularVelocity = Vector3.zero;
			}
		}

		// Handle movement
		if ( hasTarget && canMove )
		{
			transform.Translate(Vector3.forward * moveSpeed * Time.deltaTime);
			transform.eulerAngles = new Vector3(transform.eulerAngles.x, transform.eulerAngles.y, Mathf.Cos(Time.time * moveSpeed) * 15);

			if ( distance <= minFollowDistance )
			{
				rigidbody.velocity = Vector3.zero;
				rigidbody.angularVelocity = Vector3.zero;
				hasTarget = false;
			}
		}

		// Make sure we don't slide or move if we're not supposed to be moving
		if ( distance <= 3 || !hasTarget || !canMove )
		{
			rigidbody.velocity = Vector3.zero;
			rigidbody.angularVelocity = Vector3.zero;
		}
	}

	void OnMouseOver ()
	{
		if ( Input.GetButtonDown("Fire1") && Vector3.Distance(Player.transform.position, transform.position) <= minFollowDistance )
		{
			string playerText = "";
			string sheepText = "";

			switch ( Camera.main.GetComponent<GameMenu>().gameState )
			{
				case 0:
					playerText = "Peach loves the sheep!";
					sheepText = "Sheep loves the peach!";
					break;
				case 1:
					playerText = "Peach loves the sheep!";
					sheepText = "Sheep loves the peach . . .";
					break;
				case 3:
					playerText = "< I should go find sheep some flowers >\n< That will cheer her up! >";
					sheepText = "";
					break;
				default:
					playerText = "";
					sheepText = "";
					break;
			}

			Player.gameObject.SendMessage("SetTextDelay", 0f);
			Player.gameObject.SendMessage("SetTextDuration", 2.5f);
			Player.gameObject.SendMessage("UpdateText", playerText);

			gameObject.SendMessage("SetTextDelay", 3f);
			gameObject.SendMessage("SetTextDuration", 2.5f);
			gameObject.SendMessage("UpdateText", sheepText);
		}
	}

	public void ChangeSpeed(float speed)
	{
		moveSpeed = speed;
	}

	public void MoveToPoint(Vector3 newTarget)
	{
		targetPosition = newTarget;
		hasTarget = true;
	}

	public void SetTargetState(bool state)
	{
		hasTarget = state;
	}

	public void SetMoveState(bool state)
	{
		canMove = state;
	}

	public void UpdateText(string text)
	{
		if ( !showingText )
		{
			textMesh.text = text;
			StartCoroutine(DelayText(textDelay));
		}
	}

	public void SetTextDelay(float wait)
	{
		textDelay = wait;
	}

	public void SetTextDuration(float wait)
	{
		textDuration = wait;
	}

	IEnumerator DelayText(float wait)
	{
		yield return new WaitForSeconds(wait);
		showingText = true;
		StartCoroutine(RemoveText(textDuration));
	}

	IEnumerator RemoveText(float wait)
	{
		yield return new WaitForSeconds(wait);
		showingText = false;
	}
}
